/***********************************************************************************************************************
 *
 * blueBill Mobile - open source birdwatching
 * ==========================================
 *
 * Copyright (C) 2009, 2010 by Tidalwave s.a.s. (http://www.tidalwave.it)
 * http://bluebill.tidalwave.it/mobile/
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: InsertObservation.java,v f32689862d36 2010/08/17 19:14:05 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.mobile.android.test.functional;

import javax.annotation.Nonnull;
import java.util.concurrent.Callable;
import it.tidalwave.semantic.AsSupport;
import it.tidalwave.bluebill.mobile.android.taxonomy.browser.TaxonPicker;
import it.tidalwave.bluebill.mobile.android.taxonomy.browser.PickTaxonActivityTestHelper;
import it.tidalwave.bluebill.mobile.android.taxonomy.browser.TaxonomyBrowserActivityTestHelper;
import it.tidalwave.bluebill.mobile.android.location.LocationPicker;
import it.tidalwave.bluebill.mobile.android.location.PickLocationActivityTestHelper;
import it.tidalwave.bluebill.mobile.android.observation.CountAndGenderPicker;
import it.tidalwave.bluebill.mobile.android.observation.CountAndGenderActivityHelper;
import it.tidalwave.bluebill.mobile.android.observation.ObservationActivityTestHelper;
import android.test.ActivityInstrumentationTestCase2;
import com.jayway.android.robotium.solo.Solo;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: $
 *
 **********************************************************************************************************************/
public class InsertObservation extends AsSupport implements Callable<Void>
  {
    @Nonnull
    private final ObservationActivityTestHelper observationActivity;

    @Nonnull
    private final PickTaxonActivityTestHelper pickTaxonActivity;

    @Nonnull
    private final TaxonomyBrowserActivityTestHelper taxonomyBrowserActivity;

    @Nonnull
    private final CountAndGenderActivityHelper countAndGenderActivity;

    @Nonnull
    private final PickLocationActivityTestHelper pickLocationActivity;

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public InsertObservation (final @Nonnull ActivityInstrumentationTestCase2 instrumentationTestCase,
                              final @Nonnull Solo solo,
                              final @Nonnull Object ... capabilities)
      {
        super(capabilities);

        observationActivity = new ObservationActivityTestHelper(instrumentationTestCase, solo);
        pickTaxonActivity = new PickTaxonActivityTestHelper(instrumentationTestCase, solo);
        taxonomyBrowserActivity = new TaxonomyBrowserActivityTestHelper(instrumentationTestCase, solo);
        countAndGenderActivity = new CountAndGenderActivityHelper(instrumentationTestCase, solo);
        pickLocationActivity = new PickLocationActivityTestHelper(instrumentationTestCase, solo);
      }
    
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public Void call()
      throws Exception
      {
        observationActivity.waitForActivity();
        observationActivity.clickOnAddObservationButton();

        pickTaxonActivity.waitForActivity();
//        as(HistoricTaxaAsserter.class).assertHistoricTaxa(pickTaxonActivity);
        pickTaxonActivity.clickOnSpeciesButton();

        taxonomyBrowserActivity.waitForActivity();
        taxonomyBrowserActivity.waitForTaxaListNotEmpty(10000);

        as(TaxonPicker.class).pickTaxon(taxonomyBrowserActivity);
        as(CountAndGenderPicker.class).pickCountAndGender(countAndGenderActivity);
        as(LocationPicker.class).pickLocation(pickLocationActivity);

        observationActivity.waitForActivity();

        return null;
      }
  }
